# workload_identity_credentials rejects non-OIDC providers

    Code
      workload_identity_credentials(params)
    Condition
      Error in `workload_identity_credentials()`:
      ! Unsupported Workload Identity provider "UNSUPPORTED".

